@@ -25,7 +25,7 @@
 #include <projectexplorer/kitmanager.h>
 #include <projectexplorer/kit.h>
 #include <projectexplorer/kitinformation.h>
-#include <utils/projectnamevalidatinglineedit.h>
+#include <utils/projectintropage.h>
 
 #include <QCoreApplication>
 #include <QRegularExpression>
@@ -104,7 +104,7 @@ bool UbuntuDevicesModel::setData(const QModelIndex &index, const QVariant &value
         case KitListRole:
             return false;
         case DeveloperModeRole: {
-            if(!value.type() == QVariant::Bool)
+            if(value.type() != QVariant::Bool)
                 return false;
 
             bool set = value.toBool();
@@ -120,7 +120,7 @@ bool UbuntuDevicesModel::setData(const QModelIndex &index, const QVariant &value
             break;
         }
         case NetworkConnectionRole: {
-            if(!value.type() == QVariant::Bool)
+            if(value.type() != QVariant::Bool)
                 return false;
 
             bool set = value.toBool();
@@ -136,7 +136,7 @@ bool UbuntuDevicesModel::setData(const QModelIndex &index, const QVariant &value
             break;
         }
         case WriteableImageRole: {
-            if(!value.type() == QVariant::Bool)
+            if(value.type() != QVariant::Bool)
                 return false;
 
             bool set = value.toBool();
@@ -152,7 +152,7 @@ bool UbuntuDevicesModel::setData(const QModelIndex &index, const QVariant &value
             break;
         }
         case DeveloperToolsRole: {
-            if(!value.type() == QVariant::Bool)
+            if(value.type() != QVariant::Bool)
                 return false;
 
             bool set = value.toBool();
@@ -168,7 +168,7 @@ bool UbuntuDevicesModel::setData(const QModelIndex &index, const QVariant &value
             break;
         }
         case EmulatorScaleFactorRole: {
-            if(!value.type() == QVariant::String)
+            if(value.type() != QVariant::String)
                 return false;
 
             QString set = value.toString();
@@ -179,7 +179,7 @@ bool UbuntuDevicesModel::setData(const QModelIndex &index, const QVariant &value
             break;
         }
         case EmulatorMemorySettingRole: {
-            if(!value.type() == QVariant::String)
+            if(value.type() != QVariant::String)
                 return false;
 
             QString set = value.toString();
@@ -294,14 +294,6 @@ Qt::ItemFlags UbuntuDevicesModel::flags(const QModelIndex &index) const
     return QAbstractListModel::flags(index) | Qt::ItemIsEditable;
 }
 
-void UbuntuDevicesModel::triggerCloneTimeConfig(const int devId)
-{
-    int row = findDevice(devId);
-    if(row < 0)
-        return;
-    m_knownDevices[row]->device()->cloneTimeConfig();
-}
-
 void UbuntuDevicesModel::triggerPortForwarding(const int devId)
 {
     int row = findDevice(devId);
@@ -326,38 +318,6 @@ void UbuntuDevicesModel::triggerSSHConnection(const int devId)
     m_knownDevices[row]->device()->openTerminal();
 }
 
-void UbuntuDevicesModel::triggerReboot(const int devId)
-{
-    int row = findDevice(devId);
-    if(row < 0)
-        return;
-    m_knownDevices[row]->device()->reboot();
-}
-
-void UbuntuDevicesModel::triggerRebootBootloader(const int devId)
-{
-    int row = findDevice(devId);
-    if(row < 0)
-        return;
-    m_knownDevices[row]->device()->rebootToBootloader();
-}
-
-void UbuntuDevicesModel::triggerRebootRecovery(const int devId)
-{
-    int row = findDevice(devId);
-    if(row < 0)
-        return;
-    m_knownDevices[row]->device()->rebootToRecovery();
-}
-
-void UbuntuDevicesModel::triggerShutdown(const int devId)
-{
-    int row = findDevice(devId);
-    if(row < 0)
-        return;
-    m_knownDevices[row]->device()->shutdown();
-}
-
 void UbuntuDevicesModel::triggerKitAutocreate(const int devId)
 {
     int row = findDevice(devId);
@@ -373,7 +333,7 @@ void UbuntuDevicesModel::triggerKitRemove(const int devId, const QVariant &kitid
         return;
 
     ProjectExplorer::Kit* k = ProjectExplorer::KitManager::find(Core::Id::fromSetting(kitid));
-    if(ProjectExplorer::DeviceKitInformation::deviceId(k) == Core::Id(devId)) {
+    if(ProjectExplorer::DeviceKitInformation::deviceId(k) == Core::Id::fromUniqueIdentifier(devId)) {
         //completely delete the kit
         ProjectExplorer::KitManager::deregisterKit(k);
     }
@@ -823,19 +783,6 @@ void UbuntuDevicesModel::doCreateEmulatorImage(UbuntuProcess *process, const QSt
 
 void UbuntuDevicesModel::createEmulatorImage(const QString &name, const QString &arch, const QString &channel, const QString &passwd)
 {
-    QString ch = channel;
-    QString pwd = passwd;
-
-    //to work around a problem in the UITK that clips long text in dropdown menus we use short aliases in the UI
-    //for some channels. Here they are resolved to the correct channel names.
-    QMap<QString,QString> channelAliasMap = {
-        {QStringLiteral("rtm-14.09"),QStringLiteral("ubuntu-touch/ubuntu-rtm/14.09")},
-        {QStringLiteral("rtm-14.09-proposed"),QStringLiteral("ubuntu-touch/ubuntu-rtm/14.09-proposed")}
-    };
-
-    if(channelAliasMap.contains(ch))
-        ch = channelAliasMap[ch];
-
     setState(CreateEmulatorImage);
 
     //@BUG this should be cancellable but the QProcess::kill call just blocks for a long time, and then returns
@@ -843,7 +790,7 @@ void UbuntuDevicesModel::createEmulatorImage(const QString &name, const QString
     //     elevated priviledges
     setCancellable(false);
     beginAction(QString::fromLatin1(Constants::UBUNTUDEVICESWIDGET_LOCAL_CREATE_EMULATOR));
-    doCreateEmulatorImage(m_process,name,arch,ch,pwd);
+    doCreateEmulatorImage(m_process,name,arch,channel,passwd);
 }
 
 void UbuntuDevicesModel::queryAdb()
@@ -916,7 +863,7 @@ void UbuntuDevicesModel::deleteEmulator(const QString &name)
 QVariant UbuntuDevicesModel::validateEmulatorName(const QString &name)
 {
     QString error;
-    bool result = Utils::ProjectNameValidatingLineEdit::validateProjectName(name,&error);
+    bool result = Utils::ProjectIntroPage::validateProjectName(name,&error);
 
     if(result) {
         foreach (UbuntuDevicesItem *item, m_knownDevices) {
@@ -1098,7 +1045,7 @@ void UbuntuDevicesModel::processFinished(const QString &, int exitCode)
 
             //remove all ubuntu emulators that are in the settings but don't exist in the system
             foreach(int curr,notFoundImages) {
-                ProjectExplorer::DeviceManager::instance()->removeDevice(Core::Id(curr));
+                ProjectExplorer::DeviceManager::instance()->removeDevice(Core::Id::fromUniqueIdentifier(curr));
             }
 
             queryAdb();
