@@ -2,9 +2,10 @@
 #include "ui_ubuntupackagestepconfigwidget.h"
 #include "ubuntuconstants.h"
 #include "clicktoolchain.h"
-#include "ubuntuprojectguesser.h"
+#include "ubuntuprojecthelper.h"
 #include "ubuntuclickmanifest.h"
 #include "ubuntupackageoutputparser.h"
+#include "settings.h"
 
 #include <projectexplorer/kitinformation.h>
 #include <projectexplorer/target.h>
@@ -15,12 +16,12 @@
 #include <projectexplorer/ioutputparser.h>
 #include <utils/qtcprocess.h>
 #include <cmakeprojectmanager/cmakeprojectconstants.h>
+#include <qmakeprojectmanager/qmakeprojectmanagerconstants.h>
 
 #include <QDir>
 #include <QTimer>
 #include <QRegularExpression>
 #include <QVariant>
-#include <glib-2.0/glib.h>
 
 namespace Ubuntu {
 namespace Internal {
@@ -30,6 +31,7 @@ enum {
 };
 
 const char PACKAGE_MODE_KEY[] = "Ubuntu.UbuntuPackageStep.PackageMode";
+const char ERROR_MODE_KEY[] = "Ubuntu.UbuntuPackageStep.TreatErrorsAsWarnings";
 
 /*!
  * \class UbuntuPackageStep
@@ -45,9 +47,16 @@ UbuntuPackageStep::UbuntuPackageStep(ProjectExplorer::BuildStepList *bsl) :
     m_futureInterface(0),
     m_process(0),
     m_outputParserChain(0),
-    m_packageMode(EnableDebugScript)
+    m_debugMode(EnableDebugScript),
+    m_packageMode(Default),
+    m_cleanDeployDirectory(true)
 {
     setDefaultDisplayName(tr("UbuntuSDK Click build"));
+
+    m_treatClickErrorsAsWarnings = Settings::projectDefaults().reviewErrorsAsWarnings;
+
+    if (!Settings::projectDefaults().enableDebugHelper)
+        m_debugMode = DisableDebugScript;
 }
 
 UbuntuPackageStep::UbuntuPackageStep(ProjectExplorer::BuildStepList *bsl, UbuntuPackageStep *other) :
@@ -56,7 +65,9 @@ UbuntuPackageStep::UbuntuPackageStep(ProjectExplorer::BuildStepList *bsl, Ubuntu
     m_futureInterface(0),
     m_process(0),
     m_outputParserChain(0),
-    m_packageMode(other->m_packageMode)
+    m_debugMode(other->m_debugMode),
+    m_packageMode(other->m_packageMode),
+    m_cleanDeployDirectory(other->m_cleanDeployDirectory)
 {
 
 }
@@ -67,26 +78,42 @@ UbuntuPackageStep::~UbuntuPackageStep()
 }
 
 bool UbuntuPackageStep::init()
+{
+    //initialization happens in internalInit,
+    //because it requires informations that are only available at this
+    //time
+    //@TODO refactor into single buildsteps per projecttype
+    return true;
+}
+
+void UbuntuPackageStep::internalInit()
 {
     m_tasks.clear();
 
-    QString projectDir = target()->project()->projectDirectory();
-    QString deployDir;
+    QString projectDir = target()->project()->projectDirectory().toString();
+    m_buildDir.clear();
+    m_deployDir.clear();
     Utils::Environment env = Utils::Environment::systemEnvironment();
-    Utils::AbstractMacroExpander *mExp = 0;
+    Utils::MacroExpander *mExp = 0;
+
+    m_MakeParam = m_ClickParam = m_ReviewParam = ProjectExplorer::ProcessParameters();
+    m_clickPackageName.clear();
+
+    bool isCMake  = target()->project()->id() == CMakeProjectManager::Constants::CMAKEPROJECT_ID;
+    bool isQMake  = target()->project()->id() == QmakeProjectManager::Constants::QMAKEPROJECT_ID;
 
     {
-        ProjectExplorer::BuildConfiguration *bc = target()->activeBuildConfiguration();
+        ProjectExplorer::BuildConfiguration *bc = referenceBuildConfig();
         if(bc) {
             m_buildDir  = bc->buildDirectory().toString();
-            deployDir = bc->buildDirectory().toString()
+            m_deployDir = bc->buildDirectory().toString()
                     + QDir::separator()
                     + QString::fromLatin1(Constants::UBUNTU_DEPLOY_DESTDIR);
             env = bc->environment();
             mExp = bc->macroExpander();
         } else {
-            //cmake projects NEED a buildconfiguration
-            if (target()->project()->id() == CMakeProjectManager::Constants::CMAKEPROJECT_ID) {
+            //cmake and qmake projects NEED a buildconfiguration
+            if (isCMake || isQMake) {
                 ProjectExplorer::Task t(ProjectExplorer::Task::Error
                                         ,tr("No valid BuildConfiguration set for step: %1").arg(displayName())
                                         ,Utils::FileName(),-1
@@ -94,7 +121,7 @@ bool UbuntuPackageStep::init()
                 m_tasks.append(t);
 
                 //UbuntuClickPackageStep::run will stop if tasks exist
-                return true;
+                return;
             } else {
                 //backward compatibility, old HTML5 projects did not have a Buildconfiguration
                 //this would crash otherwise
@@ -102,9 +129,10 @@ bool UbuntuPackageStep::init()
                 //ubuntu + qml project types
                 QDir pDir(projectDir);
                 m_buildDir = QDir::cleanPath(target()->project()->projectDirectory()
-                                             +QDir::separator()+QStringLiteral("..")
-                                             +QDir::separator()+pDir.dirName()+QStringLiteral("_build"));
-                deployDir = m_buildDir+QDir::separator()+QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR);
+                                             .appendPath(QStringLiteral(".."))
+                                             .appendPath(pDir.dirName()+QStringLiteral("_build"))
+                                             .toString());
+                m_deployDir = m_buildDir+QDir::separator()+QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR);
 
                 //clean up the old "build"
                 QDir bd(m_buildDir);
@@ -112,14 +140,24 @@ bool UbuntuPackageStep::init()
                     bd.mkdir(m_buildDir);
             }
         }
+
+        if (!m_overrideDeployDir.isEmpty())
+            m_deployDir = m_overrideDeployDir;
+
     }
 
     //build the make process arguments
     {
-        if (target()->project()->id() == CMakeProjectManager::Constants::CMAKEPROJECT_ID) {
+        if (isCMake || isQMake) {
             QStringList arguments;
-            arguments << QStringLiteral("DESTDIR=%1").arg(QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR))
-                      << QStringLiteral("install");
+
+            if(isCMake) {
+                arguments << QStringLiteral("DESTDIR=%1").arg(m_deployDir)
+                          << QStringLiteral("install");
+            } else {
+                arguments << QStringLiteral("INSTALL_ROOT=%1").arg(m_deployDir)
+                          << QStringLiteral("install");
+            }
 
             ProjectExplorer::ProcessParameters* params = &m_MakeParam;
             params->setMacroExpander(mExp);
@@ -142,7 +180,7 @@ bool UbuntuPackageStep::init()
             //QML and HTML projects are just rsynced for now
             QStringList arguments;
             arguments << QStringLiteral("-avh")
-                      << QStringLiteral("--delete")
+                      //<< QStringLiteral("--delete")
                       << QStringLiteral("--exclude")<<QStringLiteral(".bzr")
                       << QStringLiteral("--exclude")<<QStringLiteral(".git")
                       << QStringLiteral("--exclude")<<QStringLiteral(".hg")
@@ -152,18 +190,20 @@ bool UbuntuPackageStep::init()
                       << QStringLiteral("--exclude")<<QStringLiteral("tests")
                       << QStringLiteral("--exclude")<<QStringLiteral("Makefile")
                       << QStringLiteral("--exclude")<<QStringLiteral(".excludes")
-                      << QStringLiteral("--exclude")<<QStringLiteral("*.ubuntuhtmlproject");
+                      << QStringLiteral("--exclude")<<QStringLiteral("*.ubuntuhtmlproject")
+                      << QString(QStringLiteral("--exclude-from=%1")).arg(projectDir+QDir::separator()+QStringLiteral(".excludes"));
 
-            QFile excludes (projectDir+QDir::separator()+QStringLiteral(".excludes"));
-            if (excludes.open(QIODevice::ReadOnly)) {
-                while (excludes.canReadLine()) {
-                    arguments << QStringLiteral("--exclude") << QString::fromUtf8(excludes.readLine());
-                }
-                excludes.close();
-            }
+            arguments << projectDir+QDir::separator();
 
-            arguments << projectDir+QDir::separator()
-                      << deployDir;
+            QString translationsDir = m_buildDir
+                         + QDir::separator()
+                         + QString::fromLatin1(Constants::UBUNTU_CLICK_QML_BUILD_TRANSL_DIR)
+                         + QDir::separator();
+
+            if (QDir(translationsDir).exists())
+                arguments << translationsDir;
+
+            arguments << m_deployDir;
 
             ProjectExplorer::ProcessParameters* params = &m_MakeParam;
             params->setMacroExpander(mExp);
@@ -189,13 +229,13 @@ bool UbuntuPackageStep::init()
         QStringList arguments;
         arguments << QStringLiteral("build")
                   << QStringLiteral("--no-validate")
-                  << deployDir;
+                  << m_deployDir;
 
         ProjectExplorer::ProcessParameters* params = &m_ClickParam;
         params->setMacroExpander(mExp);
 
         //setup process parameters
-        params->setWorkingDirectory(m_buildDir);
+        params->setWorkingDirectory(clickWorkingDir());
         params->setCommand(QLatin1String("click"));
         params->setArguments(Utils::QtcProcess::joinArgs(arguments));
 
@@ -214,7 +254,7 @@ bool UbuntuPackageStep::init()
         params->setMacroExpander(mExp);
 
         //setup process parameters
-        params->setWorkingDirectory(m_buildDir);
+        params->setWorkingDirectory(clickWorkingDir());
         params->setCommand(QLatin1String(Constants::CLICK_REVIEWERSTOOLS_BINARY));
 
         Utils::Environment tmpEnv = env;
@@ -224,11 +264,13 @@ bool UbuntuPackageStep::init()
         params->setEnvironment(tmpEnv);
     }
 
-    return true;
+    return;
 }
 
 void UbuntuPackageStep::run(QFutureInterface<bool> &fi)
 {
+    internalInit();
+
     if (m_tasks.size()) {
         foreach (const ProjectExplorer::Task& task, m_tasks) {
             addTask(task);
@@ -268,7 +310,7 @@ bool UbuntuPackageStep::fromMap(const QVariantMap &map)
         return false;
 
 
-    setPackageMode(AutoEnableDebugScript);
+    setDebugMode(AutoEnableDebugScript);
     if (map.contains(QLatin1String(PACKAGE_MODE_KEY))) {
         int mode = map[QLatin1String(PACKAGE_MODE_KEY)].toInt();
 
@@ -277,7 +319,10 @@ bool UbuntuPackageStep::fromMap(const QVariantMap &map)
             mode = EnableDebugScript;
 
         if(mode >= AutoEnableDebugScript && mode <= DisableDebugScript)
-            setPackageMode(static_cast<PackageMode>(mode));
+            setDebugMode(static_cast<DebugMode>(mode));
+    }
+    if (map.contains(QLatin1String(ERROR_MODE_KEY))) {
+        setTreatClickErrorsAsWarnings(map[QLatin1String(ERROR_MODE_KEY)].toBool());
     }
     return true;
 }
@@ -288,7 +333,8 @@ QVariantMap UbuntuPackageStep::toMap() const
     if(map.isEmpty())
         return map;
 
-    map.insert(QLatin1String(PACKAGE_MODE_KEY),static_cast<int>(m_packageMode));
+    map.insert(QLatin1String(PACKAGE_MODE_KEY),static_cast<int>(m_debugMode));
+    map.insert(QLatin1String(ERROR_MODE_KEY),m_treatClickErrorsAsWarnings);
     return map;
 }
 
@@ -296,20 +342,20 @@ QString UbuntuPackageStep::packagePath() const
 {
     if(m_clickPackageName.isEmpty())
         return QString();
-    return m_buildDir
+    return clickWorkingDir()
             + QDir::separator()
             + m_clickPackageName;
 }
 
-UbuntuPackageStep::PackageMode UbuntuPackageStep::packageMode() const
+UbuntuPackageStep::DebugMode UbuntuPackageStep::debugMode() const
 {
-    return m_packageMode;
+    return m_debugMode;
 }
 
-void UbuntuPackageStep::setPackageMode(UbuntuPackageStep::PackageMode arg)
+void UbuntuPackageStep::setDebugMode(UbuntuPackageStep::DebugMode arg)
 {
-    if (m_packageMode != arg) {
-        m_packageMode = arg;
+    if (m_debugMode != arg) {
+        m_debugMode = arg;
         emit packageModeChanged(arg);
     }
 }
@@ -346,13 +392,15 @@ void UbuntuPackageStep::setupAndStartProcess(const ProjectExplorer::ProcessParam
     m_process->setEnvironment(params.environment());
     m_process->setWorkingDirectory(wd.absolutePath());
 
-    if(debug) qDebug()<<"Starting process "<<params.effectiveCommand()<<params.effectiveArguments();
+    emit addOutput(tr("Starting: \"%1 %2\"").arg(params.effectiveCommand(),params.effectiveArguments()),
+                   BuildStep::MessageOutput);
 
     ProjectExplorer::IOutputParser *parser = target()->kit()->createOutputParser();
 
     //add special parser on click review step
     if(m_state == ClickReview) {
         UbuntuPackageOutputParser *packageStepParser = new UbuntuPackageOutputParser;
+        packageStepParser->setTreatAllErrorsAsWarnings(m_treatClickErrorsAsWarnings);
         connect(this,SIGNAL(currentSubStepFinished()),packageStepParser,SLOT(setEndOfData()));
         if (parser)
             parser->appendOutputParser(packageStepParser);
@@ -487,7 +535,7 @@ QString UbuntuPackageStep::makeCommand(ProjectExplorer::ToolChain *tc, const Uti
 {
     if (tc)
         return tc->makeCommand(env);
-    return QString::fromLatin1(Constants::UBUNTU_CLICK_MAKE_WRAPPER).arg(Constants::UBUNTU_SCRIPTPATH);
+    return QString();
 }
 
 /*!
@@ -497,7 +545,7 @@ QString UbuntuPackageStep::makeCommand(ProjectExplorer::ToolChain *tc, const Uti
  */
 void UbuntuPackageStep::injectDebugHelperStep()
 {
-    ProjectExplorer::BuildConfiguration *bc = target()->activeBuildConfiguration();
+    ProjectExplorer::BuildConfiguration *bc = referenceBuildConfig();
 
     if(!bc) {
         QTimer::singleShot(0,this,SLOT(doNextStep()));
@@ -505,7 +553,7 @@ void UbuntuPackageStep::injectDebugHelperStep()
     }
 
     bool ubuntuDevice = ProjectExplorer::DeviceTypeKitInformation::deviceTypeId(bc->target()->kit()).toString().startsWith(QLatin1String(Constants::UBUNTU_DEVICE_TYPE_ID));
-    bool injectDebugScript = (m_packageMode == EnableDebugScript || m_packageMode == AutoEnableDebugScript);
+    bool injectDebugScript = (m_debugMode == EnableDebugScript || m_debugMode == AutoEnableDebugScript);
 
     //debughelper script name, source and destination path
     const QString debScript = QStringLiteral("qtc_device_debughelper.py");
@@ -515,15 +563,25 @@ void UbuntuPackageStep::injectDebugHelperStep()
         QRegularExpression deskExecRegex(QStringLiteral("^(\\s*[Ee][Xx][Ee][cC]=.*)$"),QRegularExpression::MultilineOption);
 
         UbuntuClickManifest manifest;
-        if(!manifest.load(bc->buildDirectory()
-                          .appendPath(QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR))
-                          .appendPath(QStringLiteral("manifest.json"))
-                          .toString())) {
+        QString err;
+        QString manifestFileName = Utils::FileName::fromString(m_deployDir)
+                .appendPath(QStringLiteral("manifest.json")).toString();
 
+        if(!QFile::exists(manifestFileName)) {
             emit addOutput(tr("Could not find the manifest.json file in %1.\nPlease check if it is added to the install targets in your project file")
-                           .arg(bc->buildDirectory()
-                                .appendPath(QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR))
-                                .toString()),
+                           .arg(m_deployDir),
+                           BuildStep::ErrorMessageOutput);
+
+            m_futureInterface->reportResult(false);
+            cleanup();
+            emit finished();
+            return;
+        }
+
+        if(!manifest.load(manifestFileName,nullptr,&err)) {
+            emit addOutput(tr("Could not open the manifest.json file in %1.\n %2")
+                           .arg(m_deployDir)
+                           .arg(err),
                            BuildStep::ErrorMessageOutput);
 
             m_futureInterface->reportResult(false);
@@ -536,8 +594,7 @@ void UbuntuPackageStep::injectDebugHelperStep()
         foreach ( const UbuntuClickManifest::Hook &hook, hooks) {
 
             UbuntuClickManifest appArmor;
-            if (!appArmor.load(bc->buildDirectory()
-                               .appendPath(QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR))
+            if (!appArmor.load(Utils::FileName::fromString(m_deployDir)
                                .appendPath(hook.appArmorFile)
                                .toString())) {
                 qWarning()<<"Could not open the apparmor file for: "<<hook.appId;
@@ -546,14 +603,12 @@ void UbuntuPackageStep::injectDebugHelperStep()
 
             if (hook.desktopFile.isEmpty() && !hook.scope.isEmpty()) {
                 //this is a scope hook
-                const QString iniFilePath(bc->buildDirectory()
-                                          .appendPath(QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR))
+                const QString iniFilePath(Utils::FileName::fromString(m_deployDir)
                                           .appendPath(hook.scope)
                                           .appendPath(manifest.name()+QStringLiteral("_")+hook.appId+QStringLiteral(".ini"))
                                           .toString());
 
-                const QString debTargetPath = bc->buildDirectory()
-                        .appendPath(QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR))
+                const QString debTargetPath = Utils::FileName::fromString(m_deployDir)
                         .appendPath(hook.scope)
                         .appendPath(debScript).toString();
 
@@ -567,68 +622,26 @@ void UbuntuPackageStep::injectDebugHelperStep()
                 if(!injectDebugScript)
                     continue;
 
-                GKeyFile* keyFile = g_key_file_new();
-                GKeyFileFlags flags = static_cast<GKeyFileFlags>(G_KEY_FILE_KEEP_TRANSLATIONS|G_KEY_FILE_KEEP_COMMENTS);
-                if(!g_key_file_load_from_file(keyFile,qPrintable(iniFilePath),flags,NULL)){
-                    g_key_file_free(keyFile);
-                    qWarning()<<"Could not read the ini file";
+                ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(target()->kit());
+                if(!tc || tc->type() != QLatin1String(Constants::UBUNTU_CLICK_TOOLCHAIN_ID)) {
+                    qWarning()<<"Incompatible Toolchain for hook"<<hook.appId;
                     continue;
                 }
 
-                QString subCmd;
-                if(g_key_file_has_key(keyFile,"ScopeConfig","ScopeRunner",NULL)) {
-                    gchar *value = g_key_file_get_string(keyFile,"ScopeConfig","ScopeRunner",NULL);
-                    if(value == NULL) {
-                        qWarning()<<"Could not read the ScopeRunner entry";
-                        g_key_file_free(keyFile);
-                        continue;
-                    }
+                QString defaultSubCmd = QStringLiteral("/usr/lib/%1/unity-scopes/scoperunner '' %S ").arg(static_cast<ClickToolChain*>(tc)->gnutriplet());
+                QString commTemplate = QStringLiteral("./%S scope %1 %C")
+                        .arg(manifest.name()+QStringLiteral("_")+hook.appId); //tell our script the appid
 
-                    subCmd = QString::fromUtf8(value);
-                    g_free(value);
-
-                } else {
-                    ProjectExplorer::ToolChain *tc = ProjectExplorer::ToolChainKitInformation::toolChain(target()->kit());
-                    if(!tc || tc->type() != QLatin1String(Constants::UBUNTU_CLICK_TOOLCHAIN_ID)) {
-                        qWarning()<<"Incompatible Toolchain for hook"<<hook.appId;
-                        continue;
-                    }
-
-                    subCmd = QStringLiteral("/usr/lib/%1/unity-scopes/scoperunner '' %S ").arg(static_cast<ClickToolChain*>(tc)->gnutriplet());
-                }
-
-                QString command = QStringLiteral("./%1 scope %2 %3")
-                        .arg(debScript)
-                        .arg(manifest.name()+QStringLiteral("_")+hook.appId) //tell our script the appid
-                        .arg(subCmd);
-
-                g_key_file_set_string(keyFile,"ScopeConfig","ScopeRunner",command.toUtf8().data());
+                if(!UbuntuProjectHelper::injectScopeDebugHelper(iniFilePath, debScript, commTemplate, defaultSubCmd))
+                    qWarning()<<"Could not write the updated ini file";
 
                 //copy the helper script to the click package tree
                 if(QFile::exists(debSourcePath))
                     QFile::copy(debSourcePath,debTargetPath);
 
-                g_key_file_set_boolean(keyFile,"ScopeConfig","DebugMode",TRUE);
-
-                gsize size = 0;
-                gchar *settingData = g_key_file_to_data (keyFile, &size, NULL);
-                if(!settingData) {
-                    qWarning()<<"Could not convert the new data into the ini file";
-                    g_key_file_free(keyFile);
-                    continue;
-                }
-
-                gboolean ret = g_file_set_contents (qPrintable(iniFilePath), settingData, size,  NULL);
-                g_free (settingData);
-                g_key_file_free (keyFile);
-
-                if(!ret)
-                    qWarning()<<"Could not write the updated ini file";
-
             } else if(!hook.desktopFile.isEmpty() && hook.scope.isEmpty()){
 
-                const QString debTargetPath = bc->buildDirectory()
-                        .appendPath(QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR))
+                const QString debTargetPath = Utils::FileName::fromString(m_deployDir)
                         .appendPath(debScript).toString();
 
                 //make sure there is no old script in case we don't want to package it
@@ -645,8 +658,7 @@ void UbuntuPackageStep::injectDebugHelperStep()
                 //inject the debug helper into the Exec line in the desktop file
                 //@BUG if there are env vars set in the Exec line (Var=something command) this will fail
 
-                QFile deskFileFd(bc->buildDirectory()
-                                 .appendPath(QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR))
+                QFile deskFileFd(Utils::FileName::fromString(m_deployDir)
                                  .appendPath(hook.desktopFile)
                                  .toString());
 
@@ -685,7 +697,7 @@ void UbuntuPackageStep::injectDebugHelperStep()
                 continue;
             }
 
-            //if(target()->activeBuildConfiguration()->buildType() == ProjectExplorer::BuildConfiguration::Debug) {
+            //if(buildConfiguration()->buildType() == ProjectExplorer::BuildConfiguration::Debug) {
                 //enable debugging in the apparmor file, this will inject the debug policy
                 if (!appArmor.enableDebugging())
                     qWarning() <<"Could not inject debug policy, debugging with gdb will not work";
@@ -704,14 +716,42 @@ void UbuntuPackageStep::doNextStep()
         case Idle: {
             m_futureInterface->setProgressValueAndText(0,tr("Make install"));
 
-            m_state = MakeInstall;
-            setupAndStartProcess(m_MakeParam);
+            switch (m_packageMode) {
+                case OnlyClickBuild:
+                    m_state = PreparePackage;
+                    doNextStep();
+                    return;
+                case Default:
+                case OnlyMakeInstall:
+                    m_state = MakeInstall;
+
+                    if (m_cleanDeployDirectory &&
+                            //paranoid double check
+                            m_deployDir.endsWith(QDir::separator()+QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR))) {
+                        //make sure we always use a clean deploy dir
+                        QDir deplDir(m_deployDir);
+                        if(deplDir.exists())
+                            deplDir.removeRecursively();
+                    }
+
+                    setupAndStartProcess(m_MakeParam);
+                    break;
+            }
             break;
         }
         case MakeInstall: {
             if (!processFinished())
                 return;
 
+            if (m_packageMode == OnlyMakeInstall) {
+
+                m_futureInterface->reportResult(true);
+                cleanup();
+                emit finished();
+
+                return;
+            }
+
             m_futureInterface->setProgressValueAndText(1,tr("Preparing click package tree"));
             m_state = PreparePackage;
 
@@ -736,7 +776,7 @@ void UbuntuPackageStep::doNextStep()
             QRegularExpressionMatch m = exp.match(m_lastLine);
             if(m.hasMatch()) {
                 m_clickPackageName = m.captured(1);
-                emit addOutput(tr("The click package has been created in %1").arg(m_buildDir) ,
+                emit addOutput(tr("The click package has been created in %1").arg(clickWorkingDir()) ,
                                ProjectExplorer::BuildStep::MessageOutput);
             }
 
@@ -811,6 +851,80 @@ void UbuntuPackageStep::taskAdded(const ProjectExplorer::Task &task)
 {
     emit addTask(task);
 }
+bool UbuntuPackageStep::cleanDeployDirectory() const
+{
+    return m_cleanDeployDirectory;
+}
+
+void UbuntuPackageStep::setCleanDeployDirectory(bool cleanDeployDirectory)
+{
+    m_cleanDeployDirectory = cleanDeployDirectory;
+}
+
+bool UbuntuPackageStep::treatClickErrorsAsWarnings() const
+{
+    return m_treatClickErrorsAsWarnings;
+}
+
+void UbuntuPackageStep::setTreatClickErrorsAsWarnings(bool arg)
+{
+    if (m_treatClickErrorsAsWarnings != arg) {
+        m_treatClickErrorsAsWarnings = arg;
+        emit treatClickErrorsAsWarningsChanged(arg);
+    }
+}
+
+ProjectExplorer::BuildConfiguration *UbuntuPackageStep::referenceBuildConfig() const
+{
+    if(m_referenceBuildConfig)
+        return m_referenceBuildConfig.data();
+
+    return target()->activeBuildConfiguration();
+}
+
+void UbuntuPackageStep::setReferenceBuildConfig(ProjectExplorer::BuildConfiguration *referenceBuildConfig)
+{
+    m_referenceBuildConfig = referenceBuildConfig;
+}
+
+QString UbuntuPackageStep::overrideClickWorkingDir() const
+{
+    return m_overrideClickWorkingDir;
+}
+
+void UbuntuPackageStep::setOverrideClickWorkingDir(const QString &overrideClickWorkingDir)
+{
+    m_overrideClickWorkingDir = overrideClickWorkingDir;
+}
+
+QString UbuntuPackageStep::clickWorkingDir() const
+{
+    if (!m_overrideClickWorkingDir.isEmpty())
+        return m_overrideClickWorkingDir;
+    return m_buildDir;
+}
+
+QString UbuntuPackageStep::overrideInstallDir() const
+{
+    return m_overrideDeployDir;
+}
+
+void UbuntuPackageStep::setOverrideDeployDir(const QString &overrideInstallDir)
+{
+    m_overrideDeployDir = overrideInstallDir;
+}
+
+
+UbuntuPackageStep::PackageMode UbuntuPackageStep::packageMode() const
+{
+    return m_packageMode;
+}
+
+void UbuntuPackageStep::setPackageMode(const UbuntuPackageStep::PackageMode &packageMode)
+{
+    m_packageMode = packageMode;
+}
+
 
 UbuntuPackageStepConfigWidget::UbuntuPackageStepConfigWidget(UbuntuPackageStep *step) :
     SimpleBuildStepConfigWidget(step),
@@ -820,8 +934,12 @@ UbuntuPackageStepConfigWidget::UbuntuPackageStepConfigWidget(UbuntuPackageStep *
     ui->setupUi(this);
     ui->comboBoxMode->addItem(tr("Yes") ,static_cast<int>(UbuntuPackageStep::EnableDebugScript));
     ui->comboBoxMode->addItem(tr("No")  ,static_cast<int>(UbuntuPackageStep::DisableDebugScript));
-    connect(step,SIGNAL(packageModeChanged(PackageMode)),this,SLOT(updateMode()));
+
+    connect(step,SIGNAL(packageModeChanged(DebugMode)),this,SLOT(updateMode()));
+    connect(step,SIGNAL(treatClickErrorsAsWarningsChanged(bool)),this,SLOT(updateMode()));
+
     connect(ui->comboBoxMode,SIGNAL(activated(int)),this,SLOT(onModeSelected(int)));
+    connect(ui->checkBox,SIGNAL(clicked(bool)), this, SLOT(onClickErrorsToggled(bool)));
 
     updateMode();
 }
@@ -841,14 +959,17 @@ void UbuntuPackageStepConfigWidget::updateMode()
     if (m_isUpdating)
         return;
 
-    int mode = static_cast<UbuntuPackageStep*>(step())->packageMode();
-    int idx = ui->comboBoxMode->findData(mode);
+    UbuntuPackageStep *myStep = static_cast<UbuntuPackageStep*>(step());
+    int mode = myStep->debugMode();
 
-    if(idx >= 0) {
-        m_isUpdating = true;
+    m_isUpdating = true;
+    int idx = ui->comboBoxMode->findData(mode);
+    if(idx >= 0)
         ui->comboBoxMode->setCurrentIndex(idx);
-        m_isUpdating = false;
-    }
+
+    ui->checkBox->setChecked(myStep->treatClickErrorsAsWarnings());
+
+    m_isUpdating = false;
 }
 
 void UbuntuPackageStepConfigWidget::onModeSelected(const int index)
@@ -859,10 +980,19 @@ void UbuntuPackageStepConfigWidget::onModeSelected(const int index)
     int mode = ui->comboBoxMode->itemData(index).toInt();
     if ( mode >= UbuntuPackageStep::AutoEnableDebugScript && mode <= UbuntuPackageStep::DisableDebugScript ) {
         m_isUpdating = true;
-        static_cast<UbuntuPackageStep*>(step())->setPackageMode(static_cast<UbuntuPackageStep::PackageMode>(mode));
+        static_cast<UbuntuPackageStep*>(step())->setDebugMode(static_cast<UbuntuPackageStep::DebugMode>(mode));
         m_isUpdating = false;
     }
+}
 
+void UbuntuPackageStepConfigWidget::onClickErrorsToggled(const bool checked)
+{
+    if (m_isUpdating)
+        return;
+
+    m_isUpdating = true;
+    static_cast<UbuntuPackageStep*>(step())->setTreatClickErrorsAsWarnings(checked);
+    m_isUpdating = false;
 }
 
 } // namespace Internal
