@@ -18,13 +18,13 @@
 #include "ubunturemoterunconfiguration.h"
 #include "clicktoolchain.h"
 #include "ubuntuclicktool.h"
-#include "ubuntucmakebuildconfiguration.h"
 #include "ubuntuconstants.h"
 #include "ubuntulocalrunconfiguration.h"
 #include "ubunturemotedeployconfiguration.h"
 #include "ubuntupackagestep.h"
 #include "ubuntuclickmanifest.h"
 #include "ui_ubunturemoterunconfigurationwidget.h"
+#include "settings.h"
 
 #include <projectexplorer/kitinformation.h>
 #include <projectexplorer/target.h>
@@ -34,6 +34,9 @@
 #include <utils/qtcprocess.h>
 #include <cmakeprojectmanager/cmakeproject.h>
 #include <cmakeprojectmanager/cmakeprojectconstants.h>
+#include <qmakeprojectmanager/qmakeprojectmanagerconstants.h>
+#include <qmakeprojectmanager/qmakeproject.h>
+#include <qmakeprojectmanager/qmakenodes.h>
 
 #include <QFile>
 #include <QTextStream>
@@ -41,7 +44,6 @@
 #include <QVariantMap>
 #include <QVariant>
 #include <QFileInfo>
-#include <QSettings>
 
 namespace Ubuntu {
 namespace Internal {
@@ -61,6 +63,9 @@ UbuntuRemoteRunConfiguration::UbuntuRemoteRunConfiguration(ProjectExplorer::Targ
 {
     setDisplayName(appId());
     addExtraAspect(new RemoteLinux::RemoteLinuxEnvironmentAspect(this));
+
+    m_forceInstall = Settings::projectDefaults().overrideAppsByDefault;
+    m_uninstall = Settings::projectDefaults().uninstallAppsByDefault;
 }
 
 UbuntuRemoteRunConfiguration::UbuntuRemoteRunConfiguration(ProjectExplorer::Target *parent, UbuntuRemoteRunConfiguration *source)
@@ -97,6 +102,14 @@ Utils::Environment UbuntuRemoteRunConfiguration::environment() const
     QTC_ASSERT(aspect, return Utils::Environment());
     Utils::Environment env(Utils::OsTypeLinux);
     env.modify(aspect->userEnvironmentChanges());
+
+    //work around a problem with loading OPENSSL triggering a SIGILL, injecting that
+    //environment variable disables the check that causes the problem
+    if(ProjectExplorer::DeviceTypeKitInformation::deviceTypeId(target()->kit())
+            == Core::Id(Constants::UBUNTU_DEVICE_TYPE_ID).withSuffix(QStringLiteral("armhf"))) {
+        env.set(QStringLiteral("OPENSSL_armcap"),QStringLiteral("0"));
+    }
+
     return env;
 }
 
@@ -115,7 +128,23 @@ QStringList UbuntuRemoteRunConfiguration::soLibSearchPaths() const
                 paths << binary.absolutePath();
             }
         }
+    } else {
+        QmakeProjectManager::QmakeProject* qmakeProj =
+                qobject_cast<QmakeProjectManager::QmakeProject *>(target()->project());
+        if(qmakeProj) {
+            foreach (const QmakeProjectManager::QmakeProFileNode* pro, qmakeProj->allProFiles()) {
+                if(pro->projectType() == QmakeProjectManager::ApplicationTemplate
+                         || pro->projectType() == QmakeProjectManager::SharedLibraryTemplate) {
+                    QmakeProjectManager::TargetInformation info = pro->targetInformation();
+                    if(!info.valid)
+                        continue;
+                    if(debug) qDebug()<<"Adding path "<<info.buildDir<<" to solib-search-paths";
+                    paths << info.buildDir;
+                }
+            }
+        }
     }
+
     return paths;
 }
 
@@ -141,14 +170,19 @@ bool UbuntuRemoteRunConfiguration::isConfigured() const
     return true;
 }
 
+ProjectExplorer::RunConfiguration::ConfigurationState UbuntuRemoteRunConfiguration::ensureConfigured(QString *)
+{
+    return ProjectExplorer::RunConfiguration::Configured;
+}
+
 /*!
- * \brief UbuntuRemoteRunConfiguration::ensureConfigured
+ * \brief UbuntuRemoteRunConfiguration::aboutToStart
  * Configures the internal parameters and fetched the informations from
  * the manifest file. We have no way of knowing these things before the project
  * was build and all files are in <builddir>/package. That means we can not cache that
  * information, because it could change anytime
  */
-bool UbuntuRemoteRunConfiguration::ensureConfigured(QString *errorMessage)
+bool UbuntuRemoteRunConfiguration::aboutToStart(QString *errorMessage)
 {
     if(debug) qDebug()<<"--------------------- Reconfiguring RunConfiguration ----------------------------";
     m_arguments.clear();
@@ -236,15 +270,28 @@ bool UbuntuRemoteRunConfiguration::ensureConfigured(QString *errorMessage)
             m_arguments = args;
         } else {
             //looks like a application without a launcher
-            CMakeProjectManager::CMakeProject* pro = static_cast<CMakeProjectManager::CMakeProject*> (target()->project());
-            foreach(const CMakeProjectManager::CMakeBuildTarget &t, pro->buildTargets()) {
-                if(t.library || t.executable.isEmpty())
-                    continue;
+            if(target()->project()->id() == QmakeProjectManager::Constants::QMAKEPROJECT_ID) {
+                QmakeProjectManager::QmakeProject* pro = static_cast<QmakeProjectManager::QmakeProject*> (target()->project());
+                foreach(const QmakeProjectManager::QmakeProFileNode* applPro, pro->applicationProFiles()) {
+                    QmakeProjectManager::TargetInformation info = applPro->targetInformation();
+                    if(applPro->targetInformation().valid) {
+                        if(info.target == commInfo.fileName()) {
+                            m_localExecutable = info.buildDir + QDir::separator() + info.target;
+                            break;
+                        }
+                    }
+                }
+            } else {
+                CMakeProjectManager::CMakeProject* pro = static_cast<CMakeProjectManager::CMakeProject*> (target()->project());
+                foreach(const CMakeProjectManager::CMakeBuildTarget &t, pro->buildTargets()) {
+                    if(t.targetType != CMakeProjectManager::ExecutableType|| t.executable.isEmpty())
+                        continue;
 
-                QFileInfo execInfo(t.executable);
+                    QFileInfo execInfo(t.executable);
 
-                if(execInfo.fileName() == commInfo.fileName())
-                    m_localExecutable = t.executable;
+                    if(execInfo.fileName() == commInfo.fileName())
+                        m_localExecutable = t.executable;
+                }
             }
 
             if (m_localExecutable.isEmpty()) {
@@ -252,7 +299,6 @@ bool UbuntuRemoteRunConfiguration::ensureConfigured(QString *errorMessage)
                     *errorMessage = tr("Could not find %1 in the project targets").arg(command);
                 return false;
             }
-
             m_remoteExecutable = command;
         }
         return true;
@@ -268,10 +314,17 @@ bool UbuntuRemoteRunConfiguration::ensureConfigured(QString *errorMessage)
         QString manifestPath = package_dir.absoluteFilePath(QStringLiteral("manifest.json"));
 
         //read the manifest
-        UbuntuClickManifest manifest;
-        if(!manifest.load(manifestPath)) {
+        if(!QFile::exists(manifestPath)) {
             if(errorMessage)
-                *errorMessage = tr("Could not open the manifest file in the package directory, make sure its installed into the root of the click package.");
+                *errorMessage = tr("Could not find the manifest file in the package directory, make sure its installed into the root of the click package.");
+            return false;
+        }
+
+        UbuntuClickManifest manifest;
+        QString manifestErrMsg;
+        if(!manifest.load(manifestPath,nullptr,&manifestErrMsg)) {
+            if(errorMessage)
+                *errorMessage = tr("Could not read the manifest file in the package directory: %1").arg(manifestErrMsg);
             return false;
         }
 
@@ -390,18 +443,20 @@ void UbuntuRemoteRunConfiguration::setArguments(const QStringList &args)
 QString UbuntuRemoteRunConfiguration::packageDir() const
 {
     ProjectExplorer::Project *p = target()->project();
-    if (p->id() == CMakeProjectManager::Constants::CMAKEPROJECT_ID)
+    if (p->id() == CMakeProjectManager::Constants::CMAKEPROJECT_ID || p->id() == QmakeProjectManager::Constants::QMAKEPROJECT_ID)
         return target()->activeBuildConfiguration()->buildDirectory().toString()+QDir::separator()+QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR);
     else if (p->id() == Ubuntu::Constants::UBUNTUPROJECT_ID || p->id() == "QmlProjectManager.QmlProject") {
         if (!target()->activeBuildConfiguration()) {
             //backwards compatibility, try to not crash QtC for old projects
             //they did not create a buildconfiguration back then
-            QDir pDir(p->projectDirectory());
-            return p->projectDirectory()+QDir::separator()+
-                    QStringLiteral("..")+QDir::separator()+
-                    pDir.dirName()+QStringLiteral("_build")+QDir::separator()+QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR);
+            QDir pDir(p->projectDirectory().toString());
+            return p->projectDirectory()
+                    .appendPath(QStringLiteral(".."))
+                    .appendPath(pDir.dirName()+QStringLiteral("_build"))
+                    .appendPath(QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR)).toString();
         } else
-            return target()->activeBuildConfiguration()->buildDirectory().toString()+QDir::separator()+QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR);
+            return target()->activeBuildConfiguration()->buildDirectory()
+                    .appendPath(QLatin1String(Constants::UBUNTU_DEPLOY_DESTDIR)).toString();
     }
     return QString();
 }
